"""Endpoints for the GTI"""

ENDPOINT_INIT = "/gti/public/init"
ENDPOINT_CHECK_NAME = "/gti/public/checkName"
ENDPOINT_GET_ROUTE = "/gti/public/getRoute"
ENDPOINT_DEPARTURE_LIST = "/gti/public/departureList"
ENDPOINT_GET_TARIFF = "/gti/public/getTariff"
ENDPOINT_DEPARTURE_COURSE = "/gti/public/departureCourse"
ENDPOINT_LIST_STATIONS = "/gti/public/listStations"
ENDPOINT_LIST_LINES = "/gti/public/listLines"
ENDPOINT_GET_ANNOUNCEMENTS = "/gti/public/getAnnouncements"
ENDPOINT_GET_INDIVIDUAL_ROUTE = "/gti/public/getIndividualRoute"
ENDPOINT_GET_VEHICLE_MAP = "/gti/public/getVehicleMap"
ENDPOINT_GET_TRACK_COORDINATES = "/gti/public/getTrackCoordinates"
ENDPOINT_CHECK_POSTAL_CODE = "/gti/public/checkPostalCode"
ENDPOINT_GET_STATION_INFORMATION = "/gti/public/getStationInformation"
ENDPOINT_TARIFF_ZONE_NEIGHBOURS = "/gti/public/tariffZoneNeighbours"
ENDPOINT_TICKET_LIST = "/gti/public/ticketList"



import requests

from .auth import GTIAuth
from .const import *
from .schemas import *


class GTI:
    def __init__(self, username, password, server="http://api-test.geofox.de"):
        self.username = username
        self.password = password
        self.server = server

    def request(self, uri, payload):
        payload.update({"version": 37})
        res = requests.post(uri, json=payload, auth=GTIAuth(self, payload))
        return res.json()

    def init(self):
        return self.request(self.server + ENDPOINT_INIT, {})

    def checkName(self, payload):
        request = CNRequest(payload)
        return self.request(self.server + ENDPOINT_CHECK_NAME, request)

    def departureList(self, payload):
        request = DLRequest(payload)
        return self.request(self.server + ENDPOINT_DEPARTURE_LIST, request)

    def stationInformation(self, payload):
        request = SIRequest(payload)
        return self.request(self.server + ENDPOINT_GET_STATION_INFORMATION, request)



from voluptuous import Schema, Required, In

CoordinateType = In(["EPSG_4326", "EPSG_31467"])

SDType = In(["STATION", "COORDINATE", "ADDRESS", "POI", "UNKNOWN"])

FilterServiceType = In(
    [
        "ZUG",
        "UBAHN",
        "SBAHN",
        "AKN",
        "RBAHN",
        "FERNBAHN",
        "BUS",
        "STADTBUS",
        "METROBUS",
        "SCHNELLBUS",
        "NACHTBUS",
        "EILBUS",
        "AST",
        "FAEHRE",
    ]
)

TariffDetails = Schema(
    {
        "innerCity": bool,
        "cityTraffic": bool,
        "gratis": bool,
        "greaterArea": bool,
        "tariffZones": [int],
        "counties": [str],
        "rings": [str],
        "fareStage": bool,
        "fareStageNumber": int,
        "tariffNames": [str],
    }
)

Coordinate = Schema({"x": float, "y": float, "type": CoordinateType})

SDName = Schema(
    {
        "name": str,
        "city": str,
        "combinedName": str,
        "id": str,
        "type": SDType,
        "coordinate": Coordinate,
        "tariffDetails": TariffDetails,
        "serviceTypes": [str],
        "hasStationInformation": bool,
    }
)

CNRequest = Schema(
    {
        "theName": SDName,
        "maxList": int,
        "maxDistance": int,
        "coordinateType": CoordinateType,
        "tariffDetails": bool,
        "allowTypeSwitch": bool,
    }
)

GTITime = Schema({"date": str, "time": str})

FilterEntry = Schema(
    {Required("serviceID"): str, "stationIDs": [str], "serviceName": str, "label": str}
)


DLRequest = Schema(
    {
        "station": SDName,
        "stations": [SDName],
        "time": GTITime,
        "maxList": int,
        "maxTimeOffset": int,
        "allStationsInChangingNode": bool,
        "returnFilters": bool,
        "filter": [FilterEntry],
        "serviceTypes": [FilterServiceType],
        "useRealtime": bool,
    }
)

SIRequest = Schema({"station": SDName})



from requests.auth import AuthBase
import base64
import hashlib
import hmac
import json


class GTIAuth(AuthBase):
    def __init__(self, gti, payload):
        self.gti = gti
        self.sig = base64.b64encode(
            hmac.new(
                gti.password.encode("UTF-8"),
                json.dumps(payload).encode("UTF-8"),
                hashlib.sha1,
            ).digest()
        )

    def __call__(self, req):
        req.headers["geofox-auth-signature"] = self.sig
        req.headers["geofox-auth-user"] = self.gti.username
        return req
